/*
 ============================================================================
 Name		 : Engine.h
 Copyright   : Symbian Academy, 2007
 Description : Game engine declaration
 ============================================================================
 */

#ifndef ENGINE_H
#define ENGINE_H

// INCLUDES
#include <e32std.h>
#include <e32base.h>
#include <e32cons.h>            // Console

#include <e32math.h>			// Randomizer in Math Library

#include "UpdateTimer.h"

// CLASS DECLARATION
/** Street border letter. Leave a space left and right to the letter,
 * so that the old border is automatically deleted. The street can only
 * move one position to the left / right, so this is sufficient. */
_LIT(KStreetBorder, " V ");

/** Text to display when the game is over. */
_LIT(KGameOver, "Game Over");

/** Status bar text: Score, Level and Energy. */
_LIT(KStatus, "S: %d - Lvl: %d - En: %d ");

class CUpdateTimer;

/**
 *  CEngine
 * 
 */
class CEngine : public CBase
	{
public:
	// Constructors and destructor

	/**
	 * Destructor.
	 */
	~CEngine();

	/**
	 * Two-phased constructor.
	 */
	static CEngine* NewL(CConsoleBase* aConsole);

	/**
	 * Two-phased constructor.
	 */
	static CEngine* NewLC(CConsoleBase* aConsole);

private:

	/**
	 * Constructor for performing 1st stage construction
	 */
	CEngine(CConsoleBase* aConsole);

	/**
	 * EPOC default constructor for performing 2nd stage construction
	 */
	void ConstructL();

public:
	/**
	 * Set parameters for the game.
	 * Note: In the current implementation, this function can only
	 * be called once. Otherwise, it'd have to check and delete
	 * previously allocated objects.
	 * @param aEnergy maximum energy of the player
	 * @param aHeight height of the game field
	 * @param aWidth width of the game field
	 * @param aStreetWidth width of the street, should of course be smaller than the width
	 * of the game field.
	 * @param aMaxDuration how long the street can go in one direction before
	 * the game thinks about a new direction. Note that it can choose the same
	 * direction again.
	 * @param aPlayerWidth number of chars for the player.
	 */
	void SetParametersL(TInt aEnergy, TInt aHeight, TInt aWidth,
			TInt aStreetWidth, TInt aMaxDuration, TInt aPlayerWidth);

	/**
	 * Callback-function for the timer active object.
	 * It updates the world and draws it to the screen.
	 * Should be called periodically by the timer.
	 */
	void UpdateWorldL();

	/**
	 * Callback-function for the key listener active object.
	 * Updates the movement of the player.
	 * Should be called whenever the user presses a key.
	 */
	void KeyPressed(TKeyCode aKey);

	/**
	 * End the game and stop the active scheduler.
	 * This will cause the application in the main program to
	 * continue and thus exit the game.
	 */
	void GameOver(TInt aReason);

	/**
	 * Set the reference to the update timer so that the engine
	 * can increase the game speed over time.
	 */
	void SetUpdateTimer(CUpdateTimer* aUpdateTimer);

private:
	/**
	 * Calculate a new position for the street at the position
	 * that will go out of the screen in the following update.
	 */
	void NewStreetValue();

	/**
	 * Utility function to calculate a random number that's between
	 * the specified low and high value.
	 * @param aLow minimum value for the random number (included)
	 * @param aHigh maximum value for the random number (included)
	 */
	TInt GetRandomNumber(const TInt aLow, const TInt aHigh);

	/**
	 * Calculate how long the street can move into the specified 
	 * direction given the specified coordinate and the street width.
	 * e.g. if the street is near the right, this function can be used
	 * how much space is left on this side.
	 * @param aCurX current x position of the street (left side).
	 * @param aDirection direction to test: -1 for left, +1 for right.
	 */
	TInt GetMaxDurationForDirection(const TInt aCurX, const TInt aDirection);

private:
	/** Console for text output. */
	CConsoleBase* iConsole;
	/** Timer for increasing the speed. */
	CUpdateTimer* iUpdateTimer;

	/** Width of the game field. */
	TInt iWidth;
	/** Height of the game field. */
	TInt iHeight;

	/** Width of the street. Should be smaller than the game field width. */
	TInt iStreetWidth;
	/** How long the street can go in one direction before
	 * the game thinks about a new direction. Note that it can choose the same
	 * direction again. */
	TInt iMaxDuration;

	/** The array containing the screen positions is organized as a circular
	 * array to prevent shuffling the elements around. This variable stores
	 * the current end position within the array. */
	TInt iCurArrayPos;
	/** Array containing the left x-coordinates of the street for the individual
	 * lines. */
	TInt* iStreetPos;
	/** Current direction of the street. -1: left, 0: straight ahead, +1: right. */
	TInt iCurDirection;
	/** For how many updates (= how many more lines) should the street continue
	 * in the current direction. When the time is over, the game will think of a
	 * new direction. */
	TInt iDurationLeft;

	/** Current x-coordinate (left side) of the player. */
	TInt iPlayerPos;
	/** x-coordinate of the player at the last update. Important because it can be
	 * that two or more movements happen between two successive screen updates.
	 * The last position of the player has to be cleared in that case. */
	TInt iLastPlayerPos;
	/** Number of chars for the player. */
	TInt iPlayerWidth;
	/** Descriptor containing the player in the specified size. */
	RBuf iPlayerGraphics;
	/** Line containing only spaces with the width of the play field. Parts of this
	 * descriptor are used to clear the game field. */
	RBuf iSpaceLine;

	/** Statistics: Current energy of the player. 0 -> game over. */
	TInt iEnergy;
	/** Current score of the player. Increased with every update of the game. */
	TInt iScore;
	/** Current level of the game. Increased every 100 points, also makes the game faster. */
	TInt iLevel;
	};

#endif // ENGINE_H

